@@ -27,15 +27,23 @@
 #include "pythonsession.h"
 #include "pythonkeywords.h"
 
+using namespace Cantor;
+
 PythonCompletionObject::PythonCompletionObject(const QString& command, int index, PythonSession* session) : Cantor::CompletionObject(session),
 m_expression(nullptr)
 {
     setLine(command, index);
 }
 
+PythonCompletionObject::~PythonCompletionObject()
+{
+    if (m_expression)
+        m_expression->setFinishingBehavior(Expression::DeleteOnFinish);
+}
+
 void PythonCompletionObject::fetchCompletions()
 {
-    if (session()->status() == Cantor::Session::Disable)
+    if (session()->status() != Session::Done)
     {
         QStringList allCompletions;
 
@@ -59,9 +67,7 @@ void PythonCompletionObject::fetchCompletions()
             "print('|'.join(rlcompleter.Completer(__dict__).global_matches('%1')+rlcompleter.Completer(__dict__).attr_matches('%1')))"
         ).arg(command());
         m_expression = session()->evaluateExpression(expr, Cantor::Expression::FinishingBehavior::DoNotDelete, true);
-        // TODO: Python exec the expression before connect, so manualy run handler. Uncomment the connection after removing DBus
-        // connect(m_expression, &Cantor::Expression::statusChanged, this, &PythonCompletionObject::extractCompletions);
-        extractCompletions(m_expression->status());
+        connect(m_expression, &Cantor::Expression::statusChanged, this, &PythonCompletionObject::extractCompletions);
     }
 }
 
@@ -69,7 +75,7 @@ void PythonCompletionObject::fetchCompletions()
 
 void PythonCompletionObject::fetchIdentifierType()
 {
-    if (session()->status() == Cantor::Session::Disable)
+    if (session()->status() != Cantor::Session::Done)
     {
         if (qBinaryFind(PythonKeywords::instance()->functions().begin(),
                 PythonKeywords::instance()->functions().end(), identifier())
@@ -89,17 +95,12 @@ void PythonCompletionObject::fetchIdentifierType()
 
         const QString& expr = QString::fromLatin1("callable(%1)").arg(identifier());
         m_expression = session()->evaluateExpression(expr, Cantor::Expression::FinishingBehavior::DoNotDelete, true);
-        // TODO: Python exec the expression before connect, so manualy run handler. Uncomment the connection after removing DBus
-        // connect(m_expression, &Cantor::Expression::statusChanged, this, &PythonCompletionObject::extractIdentifierType);
-        extractIdentifierType(m_expression->status());
+        connect(m_expression, &Cantor::Expression::statusChanged, this, &PythonCompletionObject::extractIdentifierType);
     }
 }
 
 void PythonCompletionObject::extractCompletions(Cantor::Expression::Status status)
 {
-    if (!m_expression)
-        return;
-
     switch(status)
     {
         case Cantor::Expression::Error:
@@ -124,8 +125,6 @@ void PythonCompletionObject::extractCompletions(Cantor::Expression::Status statu
 
 void PythonCompletionObject::extractIdentifierType(Cantor::Expression::Status status)
 {
-    if (!m_expression)
-            return;
     switch(status)
     {
         case Cantor::Expression::Error:
@@ -133,11 +132,15 @@ void PythonCompletionObject::extractIdentifierType(Cantor::Expression::Status st
             if (m_expression->errorMessage().contains(QLatin1String("SyntaxError: invalid syntax")))
                 emit fetchingTypeDone(KeywordType);
             else
+            {
                 qDebug() << "Error with PythonCompletionObject" << (m_expression->result() ? m_expression->result()->toHtml() : QLatin1String("extractIdentifierType"));
+                emit fetchingTypeDone(UnknownType);
+            }
             break;
 
         case Cantor::Expression::Interrupted:
             qDebug() << "PythonCompletionObject was interrupted";
+            emit fetchingTypeDone(UnknownType);
             break;
 
         case Cantor::Expression::Done:
