@@ -27,15 +27,29 @@ class TestPython3 : public BackendTest
 {
   Q_OBJECT
   private Q_SLOTS:
-    void testImportNumpy();
+    void testSimpleCommand();
+    void testMultilineCommand();
     void testCodeWithComments();
-    void testPython3Code();
+    void testCommandQueue();
+
     void testSimplePlot();
 
+    void testImportStatement();
+    void testPython3Code();
+    void testInvalidSyntax();
+
+    void testSimpleExpressionWithComment();
+    void testCommentExpression();
+    void testMultilineCommandWithComment();
+
     void testVariablesCreatingFromCode();
     void testVariableCleanupAfterRestart();
     void testDictVariable();
 
+    void testCompletion();
+    void testInterrupt();
+
+    void testWarning();
   private:
     QString backendName() override;
 };
