@@ -27,6 +27,7 @@
 #include <KStandardAction>
 #include <KNS3/DownloadDialog>
 #include <KParts/ReadWritePart>
+#include <KRecentFilesAction>
 
 #include <QApplication>
 #include <QCloseEvent>
@@ -48,7 +49,6 @@
 
 CantorShell::CantorShell() : KParts::MainWindow(), m_part(nullptr)
 {
-
     // set the shell's ui resource file
     setXMLFile(QLatin1String("cantor_shell.rc"));
 
@@ -76,6 +76,12 @@ CantorShell::CantorShell() : KParts::MainWindow(), m_part(nullptr)
     updateNewSubmenu();
 }
 
+CantorShell::~CantorShell()
+{
+    if (m_recentProjectsAction)
+        m_recentProjectsAction->saveEntries(KSharedConfig::openConfig()->group(QLatin1String("Recent Files")));
+}
+
 void CantorShell::load(const QUrl &url)
 {
     if (!m_part||!m_part->url().isEmpty() || m_part->isModified() )
@@ -83,7 +89,10 @@ void CantorShell::load(const QUrl &url)
         addWorksheet(QLatin1String("null"));
         m_tabWidget->setCurrentIndex(m_parts.size()-1);
     }
-    m_part->openUrl( url );
+    if (!m_part->openUrl( url ))
+        closeTab(m_tabWidget->currentIndex());
+    if (m_recentProjectsAction)
+        m_recentProjectsAction->addUrl(url);
 }
 
 bool CantorShell::hasAvailableBackend()
@@ -104,6 +113,9 @@ void CantorShell::setupActions()
     openNew->setPriority(QAction::LowPriority);
     QAction* open = KStandardAction::open(this, SLOT(fileOpen()), actionCollection());
     open->setPriority(QAction::LowPriority);
+    m_recentProjectsAction = KStandardAction::openRecent(this, &CantorShell::load, actionCollection());
+    m_recentProjectsAction->setPriority(QAction::LowPriority);
+    m_recentProjectsAction->loadEntries(KSharedConfig::openConfig()->group(QLatin1String("Recent Files")));
 
     KStandardAction::close (this,  SLOT(closeTab()),  actionCollection());
 
@@ -285,7 +297,7 @@ void CantorShell::addWorksheet(const QString& backendName)
     // this routine will find and load our Part.  it finds the Part by
     // name which is a bad idea usually.. but it's alright in this
     // case since our Part is made for this Shell
-    KPluginFactory* factory = KPluginLoader(QLatin1String("libcantorpart")).factory();
+    KPluginFactory* factory = KPluginLoader(QLatin1String("cantorpart")).factory();
     if (factory)
     {
         Cantor::Backend* backend = Cantor::Backend::getBackend(backendName);
@@ -299,6 +311,7 @@ void CantorShell::addWorksheet(const QString& backendName)
                 if (part)
                 {
                     connect(part, SIGNAL(setCaption(QString,QIcon)), this, SLOT(setTabCaption(QString,QIcon)));
+                    connect(part, SIGNAL(worksheetSave(QUrl)), this, SLOT(onWorksheetSave(QUrl)));
                     m_parts.append(part);
 
                     int tab = m_tabWidget->addTab(part->widget(), QIcon::fromTheme(backend->icon()), i18n("Session %1", sessionCount++));
@@ -595,19 +608,26 @@ void CantorShell::updatePanel()
         addDockWidget ( Qt::RightDockWidgetArea,  docker );
 
         // Set visibility for dock from saved info
-        if (!isNewWorksheet)
+        if (isNewWorksheet)
+        {
+            if (plugin->showOnStartup())
+                docker->show();
+            else
+                docker->hide();
+        }
+        else
+        {
             if (m_pluginsVisibility[m_part].contains(plugin->name()))
                 docker->show();
             else
                 docker->hide();
-        else
-            docker->show();
+        }
 
         if(last!=nullptr)
             tabifyDockWidget(last, docker);
         last=docker;
 
-        connect(plugin, SIGNAL(visibilityRequested()), docker, SLOT(raise()));
+        connect(plugin, &Cantor::PanelPlugin::visibilityRequested, this, &CantorShell::pluginVisibilityRequested);
 
         m_panels.append(docker);
 
@@ -646,3 +666,23 @@ Cantor::WorksheetAccessInterface* CantorShell::currentWorksheetAccessInterface()
 
     return wa;
 }
+
+void CantorShell::pluginVisibilityRequested()
+{
+    Cantor::PanelPlugin* plugin = static_cast<Cantor::PanelPlugin*>(sender());
+    for (QDockWidget* docker: m_panels)
+    {
+        if (plugin->name() == docker->windowTitle())
+        {
+            if (docker->isHidden())
+                docker->show();
+            docker->raise();
+        }
+    }
+}
+
+void CantorShell::onWorksheetSave(const QUrl& url)
+{
+    if (m_recentProjectsAction)
+        m_recentProjectsAction->addUrl(url);
+}
