@@ -500,6 +500,14 @@ QDomElement CommandEntry::toXml(QDomDocument& doc, KZip* archive)
 
     // save results of the expression if they exist
     if (expression())
+    {
+        const QString& errorMessage = expression()->errorMessage();
+        if (!errorMessage.isEmpty())
+        {
+            QDomElement errorElem = doc.createElement( QLatin1String("Error") );
+            errorElem.appendChild(doc.createTextNode(errorMessage));
+            exprElem.appendChild(errorElem);
+        }
         for (Cantor::Result * const result: expression()->results())
         {
             const QDomElement& resultElem = result->toXml(doc);
@@ -508,6 +516,7 @@ QDomElement CommandEntry::toXml(QDomDocument& doc, KZip* archive)
             if (archive)
                 result->saveAdditionalData(archive);
         }
+    }
 
     //save the background color if it differs from the default one
     const QColor& backgroundColor = m_commandItem->backgroundColor();
@@ -676,7 +685,14 @@ void CommandEntry::expressionChangedStatus(Cantor::Expression::Status status)
         }
 
         if (status == Cantor::Expression::Error)
-            m_errorItem->setHtml(m_expression->errorMessage());
+        {
+            QString error = m_expression->errorMessage().toHtmlEscaped();
+            while (error.endsWith(QLatin1Char('\n')))
+                error.chop(1);
+            error.replace(QLatin1String("\n"), QLatin1String("<br>"));
+            error.replace(QLatin1String(" "), QLatin1String("&nbsp;"));
+            m_errorItem->setHtml(error);
+        }
         else
             m_errorItem->setHtml(i18n("Interrupted"));
 
